/*
 * braid.h -- A class for creating and drawing (mathematical) braids in ePiX.
 *
 * Last Change: October 15, 2007
 */

/*
 * Copyright (C) 2007
 * Andrew D. Hwang <ahwang -at- mathcs -dot- holycross -dot- edu>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "epix.h"

namespace ePiX {

  class braid {

  public:
    braid(unsigned int strands);

    // strands numbered 0, 1, ..., n-1
    braid& twist(unsigned int i1, unsigned int i2, int twists=1);

    // Draw open strands in coordinate rectangle w/specified corners.
    // Stands oriented as follows:
    // Horizontal                      Vertical
    //  *----pt2                       pt1----*
    //  |-----|                         |  |  |
    // pt1----*                         *----pt2
    //
    void draw(const P& pt1, const P& pt2) const;

    // draw strands as counter-clockwise loops in specified annulus
    void draw(const P& ctr, double inner, double outer) const;

  private:
    class twist_data {
    public:
      twist_data(unsigned int i1, unsigned int i2);

      unsigned int first() const;
      unsigned int second() const;

    private:
      unsigned int m_strand1;
      unsigned int m_strand2;
    }; // end of class twist_data

    unsigned int m_strands;
    std::list<twist_data> m_twists;
  }; // end of class braid
}
